//----------------------------------------------------------------------------
// ObjectWindows - (C) Copyright 1991, 1993 by Borland International
//----------------------------------------------------------------------------
#include <owl\owlpch.h>
#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\editfile.h>
#include <cstring.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <idapi.h>
#include "informnt.rh"

#define MAXLEN 100
#define RESULTFILE "results.bde"

class TFileApp : public TApplication {
  public:
    hDBIDb          hDb;
    int             hFile;    // file for output
    

    TFileApp() : TApplication ("IDAPI Informant") {}
    
    void       WriteSchemaTable  (hDBICur hCur, UINT32 uDisplayNRecs);
    void       WriteOut          (int hFile, char *pszText, int nBytes);
    DBIResult  ChkRslt           (DBIResult rslt, pCHAR pMsg);
    DBIResult  GetFields         (hDBICur hCur, pFLDDesc pFldDescs, pBYTE pRecBuf,
                                  CHAR** pszField, UINT16 uDefLength);
    DBIResult  SetupFields       (CURProps TblProps, pFLDDesc pFldDescs, 
                                  CHAR** pszField, UINT16 uDefLength);
    DBIResult  FormatDate        (DATE Date, pCHAR szDate);
    DBIResult  FormatTime        (TIME Time, pCHAR szTime);
                        
  protected:
    void   InitMainWindow ();
    void   CmFileNew ();
    void   CmFileOpen ();
    void   CmIDAPITableList ();
    void   CmIDAPIExtTableList ();
    void   CmIDAPIFileList ();
    void   CmIDAPIDriverList ();
    void   CmIDAPIUserList ();
    void   CmIDAPIDatabaseList ();
    void   CmIDAPILdList ();
    void   InitIDAPI (void);                 
                
  DECLARE_RESPONSE_TABLE (TFileApp);
};

DEFINE_RESPONSE_TABLE1(TFileApp, TApplication)
  EV_COMMAND (CM_FILENEW, CmFileNew),
  EV_COMMAND (CM_FILEOPEN, CmFileOpen),
  EV_COMMAND (CM_TABLELIST, CmIDAPITableList),
  EV_COMMAND (CM_EXTTABLELIST, CmIDAPIExtTableList),
  EV_COMMAND (CM_FILELIST, CmIDAPIFileList),
  EV_COMMAND (CM_DRIVERLIST, CmIDAPIDriverList),
  EV_COMMAND (CM_USERLIST, CmIDAPIUserList),
  EV_COMMAND (CM_DATABASELIST, CmIDAPIDatabaseList),
  EV_COMMAND (CM_LDLIST, CmIDAPILdList),
END_RESPONSE_TABLE;         


//
// Construct the TFileApp's MainWindow's client of type TEditFile
//
void
TFileApp::InitMainWindow()
{
   MainWindow = new TFrameWindow(0, Name, new TEditFile);
   MainWindow->AssignMenu(IDM_INFORMANT);
   MainWindow->Attr.AccelTable = IDA_EDITFILE;
   MainWindow->SetIcon(this, 201);

   InitIDAPI ();            
}                 


void TFileApp::InitIDAPI ()
{
   DBIResult rslt;

   rslt = DbiInit (NULL);
   if (ChkRslt (rslt, "DbiInit"))
      return;
   rslt = DbiOpenDatabase (NULL, NULL, dbiREADWRITE, dbiOPENSHARED, NULL, 0, NULL,
         NULL, &hDb);
   ChkRslt (rslt, "DbiOpenDatabase");
}         


void
TFileApp::CmFileNew()
{
  TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
  CHECK(ef);
  ef->NewFile();
}

void
TFileApp::CmFileOpen()
{
  TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
  CHECK(ef);
  ef->Open();
}

void
TFileApp::CmIDAPITableList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenTableList (hDb, FALSE, FALSE, NULL, &hCur);
   if (ChkRslt (rslt, "DbiOpenTableList"))
      return;
   
   WriteSchemaTable (hCur, 100);

  
   // show file on screen
   TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
   CHECK(ef);
   ef->Read (RESULTFILE);                     
}

void
TFileApp::CmIDAPIExtTableList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenTableList (hDb, TRUE, FALSE, NULL, &hCur);
   if (ChkRslt (rslt, "DbiOpenTableList"))
      return;
   
   WriteSchemaTable (hCur, 100);

  
   // show file on screen
   TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
   CHECK(ef);
   ef->Read (RESULTFILE);                     
}



void
TFileApp::CmIDAPIFileList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenFileList (hDb, "*.*", &hCur);
   if (ChkRslt (rslt, "DbiOpenFileList"))
      return;
      
   WriteSchemaTable (hCur, 100);

   // show file on screen
   TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
   CHECK(ef);
   ef->Read (RESULTFILE);                     
}

void
TFileApp::CmIDAPIDriverList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenDriverList (&hCur);
   if (ChkRslt (rslt, "OpenDriverList"))
      return;
      
   WriteSchemaTable (hCur, 100);

   // show file on screen
   TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
   CHECK(ef);
   ef->Read (RESULTFILE);                     
}

void
TFileApp::CmIDAPIUserList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenUserList (&hCur);
   if (ChkRslt (rslt, "DbiOpenUserList"))
      return;

   WriteSchemaTable (hCur, 100);

  // show file on screen
  TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
  CHECK(ef);
  ef->Read (RESULTFILE);                     
}

void
TFileApp::CmIDAPIDatabaseList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenDatabaseList (&hCur);
   if (ChkRslt (rslt, "DbiOpenDatabaseList"))
      return;
      
   WriteSchemaTable (hCur, 100);

   // show file on screen
   TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
   CHECK(ef);
   ef->Read (RESULTFILE);                     
}


void
TFileApp::CmIDAPILdList ()
{
   DBIResult rslt;
   hDBICur   hCur;
   
   // write to file
   rslt = DbiOpenLdList (&hCur);
   if (ChkRslt (rslt, "DbiOpenDatabaseList"))
      return;
      
   WriteSchemaTable (hCur, 100);

  // show file on screen
  TEditFile* ef = TYPESAFE_DOWNCAST(MainWindow->GetClientWindow(), TEditFile);
  CHECK(ef);
  ef->Read (RESULTFILE);                     
}


                           
void TFileApp::WriteSchemaTable (hDBICur hCur, UINT32 uDisplayNRecs)
{
    DBIResult   rslt;               // IDAPI function return value
    CURProps    TblProps;           // Table Properties
    pFLDDesc    pFldDescs = 0;      // List of fields
    pBYTE       pRecBuf = 0;        // Record Buffer
    CHAR**      pszField = 0;       // Array to contain the fields of the
                                    // table.
    CHAR*       pszFormat;          // Contains an entire record (row)
    CHAR        szTemp[100] ="";    // Temporary variable for reading data
    UINT16      uFldNum;            // Loop variable
    UINT32      uCurCountRec=0;     // Counter to keep track of how many
                                    // records have been displayed
    const UINT16 uDefLength = 15;   // Default size of a field
    int         nBytes = 0;


    // Open output file
    unlink (RESULTFILE);
    hFile = open (RESULTFILE, O_CREAT | O_APPEND, S_IWRITE);

    pszFormat = (pCHAR) malloc(1600 * sizeof(BYTE));
    if (pszFormat == NULL)
    {
        WriteOut (hFile, "    Error - Out of memory", 0);
        goto EXIT;
    }

    rslt = DbiGetCursorProps(hCur, &TblProps);
    if (ChkRslt(rslt, "GetCursorProps") != DBIERR_NONE)
    {
        free(pszFormat);
        goto EXIT;
    }

    // Allocate space for the record buffer
    pRecBuf = (pBYTE) malloc(TblProps.iRecBufSize * sizeof(BYTE));
    if (pRecBuf == NULL)
    {
        WriteOut (hFile, "    Error - Out of memory", 0);
        goto EXIT;
    }

    // Allocate enough space to contain information (Field Descriptors)
    // about all the fields in the answer table.
    pFldDescs = (pFLDDesc) malloc(TblProps.iFields * sizeof(FLDDesc));
    if (pFldDescs == NULL)
    {
       free(pszFormat);
       free(pRecBuf);
       WriteOut(hFile, "    Error - Out of memory", 0);
       goto EXIT;
    }

    // Get information about the fields.
    rslt = DbiGetFieldDescs(hCur, pFldDescs);
    ChkRslt(rslt, "GetFieldDescs");

    // Allocate enough buffers to contain data from the fields in the
    // answer table. Also sets the width of non-fldZSTRING fields
    // (all data will be converted to strings for display )
    pszField = (CHAR**) malloc(TblProps.iFields * sizeof(CHAR*));

    // Set up formatting for the fields
    SetupFields(TblProps, pFldDescs, pszField, uDefLength);

    // Display the names of the fields, aligned by column
    strcpy(pszFormat,"\n ");
    for (uFldNum = 0; uFldNum < TblProps.iFields; uFldNum++)
    {
        sprintf(szTemp, "%-*s\t", 15, pFldDescs[uFldNum].szName);
        strcat(pszFormat, szTemp);
        nBytes += strlen (szTemp);
    }

    // Display the header
    WriteOut(hFile, pszFormat, nBytes);

    // Get records from the table until the end of the table is reached
    // or the maximum number of records is reached.
    // Note that not all field types are supported. All supported field
    // types are displayed as strings.
    if (uDisplayNRecs == 0)
    {
        uDisplayNRecs = (UINT16)-1; // uDisplayNRecs is unsigned, so
                                    // -1 maximizes the value
    }

    while ((uCurCountRec < uDisplayNRecs) &&
           ((rslt = DbiGetNextRecord(hCur, dbiNOLOCK, pRecBuf, NULL))
            == DBIERR_NONE))
    {
        nBytes = 0;
        // Fill the record buffer with the field values
        uCurCountRec++;
        GetFields(hCur, pFldDescs, pRecBuf, pszField, uDefLength);
        
        // Initialize pszFormat to all zeroes
        memset(pszFormat, 0, sizeof(pszFormat));

        // Add leading blank space to the record
        strcpy(pszFormat," ");

        // Add each field to be displayed, making certain that the
        // columns line up.
        for (uFldNum = 0; uFldNum < TblProps.iFields; uFldNum++)
        {
            sprintf(szTemp, "%-*s\t", 17, pszField[uFldNum]);
            strcat(pszFormat, szTemp);
            nBytes += strlen (szTemp);
        }

        // Display the record
        WriteOut(hFile, pszFormat, nBytes);
    }

    // Expect the End-Of-File error - just means that there
    // are no more records in the table.
    if (rslt == DBIERR_EOF)
        rslt = DBIERR_NONE;

EXIT:
    if (hFile != -1)
       hFile = close (hFile);
    
    // Clean up
    for (uFldNum = 0; uFldNum < TblProps.iFields; uFldNum++)
    {
        if (pszField[uFldNum])
           free(pszField[uFldNum]);
    }

    if (pszField)
       free(pszField);
    if (pFldDescs)
       free(pFldDescs);
    if (pRecBuf)
       free(pRecBuf);
    if (pszFormat)
       free(pszFormat);

    return;
}

void TFileApp::WriteOut (int hFile, char *pszText, int nBytes)
{
   if (nBytes == 0)
     nBytes = strlen (pszText);
     
   write (hFile, pszText, nBytes);
   write (hFile, "\x0d\x0a", 2);
}


DBIResult
TFileApp::ChkRslt (DBIResult rslt, pCHAR pMsg)
{
    DBIMSG  dbi_status;
    DBIErrInfo  ErrInfo;
    CHAR    buf [255];

    // Echo only if actual result doesn't equal expected result
    if (rslt != DBIERR_NONE)
    {
        // Get as much error information as possible
        DbiGetErrorInfo(TRUE, &ErrInfo);

        // Make certain information is returned on the correct error
        if (ErrInfo.iError == rslt)
        {
            sprintf (buf, "ERROR -  %s"
                   "\r\n                 Cat:Code = [%xh:%xh]"
                   "\r\n                 %s",
                   pMsg, ErrCat(ErrInfo.iError), ErrCode(ErrInfo.iError),
                   ErrInfo.szErrCode);

            if (strcmp(ErrInfo.szContext1, ""))
            {
                strcat(buf, ErrInfo.szContext1);
            }
            if (strcmp(ErrInfo.szContext2, ""))
            {
                strcat(buf, ErrInfo.szContext2);
            }
            if (strcmp(ErrInfo.szContext3, ""))
            {
                strcat(buf, ErrInfo.szContext3);
            }
            if (strcmp(ErrInfo.szContext4, ""))
            {
                strcat(buf, ErrInfo.szContext4);
            }
        }
        else {
            DbiGetErrorString(rslt, dbi_status);
            sprintf (buf, "ERROR - %s  Cat:Code [%xh:%xh]\r\n"
                   "                %s",
                   pMsg, ErrCat(rslt), ErrCode(rslt), dbi_status);
        }
        MainWindow->MessageBox (buf, "Error", MB_OK);
    }
    return rslt;
}


DBIResult
TFileApp::SetupFields (CURProps TblProps, pFLDDesc pFldDescs, CHAR** pszField,
             UINT16 uDefLength)
{
    UINT16      uFldNum;            // Loop variable

    for (uFldNum = 0; uFldNum < TblProps.iFields; uFldNum++)
    {
        // Allocate enough space to contain the data in each field
        switch (pFldDescs[uFldNum].iFldType)
        {
            case fldBYTES:
            case fldZSTRING:
                pszField[uFldNum] = (CHAR*)malloc((pFldDescs[uFldNum].iLen
                                                  * sizeof(CHAR)));
                if (MAXLEN - 4 < pFldDescs[uFldNum].iLen)
                {
                    pFldDescs[uFldNum].iUnits1 = MAXLEN-4;
                    pFldDescs[uFldNum].iLen    = MAXLEN-3;
                }
                // Adjust if the field is smaller than its description
                if (pFldDescs[uFldNum].iLen <
                    strlen(pFldDescs[uFldNum].szName))
                {
                    pFldDescs[uFldNum].iUnits1 =
                                        strlen(pFldDescs[uFldNum].szName);
                    pFldDescs[uFldNum].iLen    =
                                        pFldDescs[uFldNum].iUnits1 - 1;
                }
                break;
            case fldFLOAT:
            case fldDATE:
            case fldINT16:
            case fldUINT16:
            case fldINT32:
            case fldUINT32:
            case fldBOOL:
            case fldBLOB:
            case fldBCD:
                pszField[uFldNum] = (CHAR*)malloc((uDefLength *
                                                  sizeof(CHAR)));
                // Set the width of the field as it will be displayed
                pFldDescs[uFldNum].iUnits1 = uDefLength-1;
                pFldDescs[uFldNum].iLen    = uDefLength;
                // Adjust if the field is smaller than it's description
                if (pFldDescs[uFldNum].iLen <
                    strlen(pFldDescs[uFldNum].szName))
                {
                    pFldDescs[uFldNum].iUnits1 =
                                        strlen(pFldDescs[uFldNum].szName);
                    pFldDescs[uFldNum].iLen    =
                                        pFldDescs[uFldNum].iUnits1 - 1;
                }
                break;
            case fldTIMESTAMP:
                pszField[uFldNum] = (CHAR*)malloc((36 *
                                                  sizeof(CHAR)));
                // Set the width of the field as it will be displayed
                pFldDescs[uFldNum].iUnits1 = 36;
                pFldDescs[uFldNum].iLen    = 35;
                // Adjust if the field is smaller than it's description
                if (pFldDescs[uFldNum].iLen <
                    strlen(pFldDescs[uFldNum].szName))
                {
                    pFldDescs[uFldNum].iUnits1 =
                                        strlen(pFldDescs[uFldNum].szName);
                    pFldDescs[uFldNum].iLen    =
                                        pFldDescs[uFldNum].iUnits1 - 1;
                }
                break;
            case fldTIME:
                pszField[uFldNum] = (CHAR*)malloc((20 *
                                                  sizeof(CHAR)));
                // Set the width of the field as it will be displayed
                pFldDescs[uFldNum].iUnits1 = 20;
                pFldDescs[uFldNum].iLen    = 19;

                // Adjust if the field is smaller than its description
                if (pFldDescs[uFldNum].iLen <
                    strlen(pFldDescs[uFldNum].szName))
                {
                    pFldDescs[uFldNum].iUnits1 =
                                        strlen(pFldDescs[uFldNum].szName);
                    pFldDescs[uFldNum].iLen    =
                                        pFldDescs[uFldNum].iUnits1 - 1;
                }
              break;
            default:
                // This field size will hold the 'Not Supported'
                // message
                pszField[uFldNum] = (CHAR*)malloc((uDefLength *
                                                  sizeof(CHAR)));
                // Set the width of the field as it will be displayed
                pFldDescs[uFldNum].iUnits1 = uDefLength;
                break;
        }
    }

    return DBIERR_NONE;
}


DBIResult
TFileApp::GetFields (hDBICur hCur, pFLDDesc pFldDescs, pBYTE pRecBuf,
           CHAR ** pszField, UINT16 uDefLength)
{
    DBIResult   rslt;           // Return value from IDAPI functions
    CURProps    TblProps;       // Properties of the table
    UINT16      uFldNum;        // Loop Counter - used to determine the
                                // current field.
    CHAR        szTemp[MAXLEN] =""; // Temporary variable for reading data
    UINT32      ulBlobSize;
    FLOAT       lfFloat;

    BOOL        bIsBlank;           // Check if the field is blank

    rslt = DbiGetCursorProps(hCur, &TblProps);
    if (ChkRslt(rslt, "GetCursorProps") != DBIERR_NONE)
    {
        return rslt;
    }

    // Get the field values from the record. Gets each field from
    // the table, placing the results in the pszField variable
    for (uFldNum = 0; uFldNum < TblProps.iFields; uFldNum++)
    {
        switch (pFldDescs[uFldNum].iFldType)
        {
            case fldBYTES:
            case fldZSTRING:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) pszField[uFldNum],
                                   &bIsBlank);
                ChkRslt(rslt, "GetField");
                break;
            case fldFLOAT:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    if (pFldDescs[uFldNum].iSubType == fldstMONEY)
                    {
                        sprintf(pszField[uFldNum], "$%.2lf",
                                *(FLOAT *)szTemp);
                    }
                    else
                    {
                        sprintf(pszField[uFldNum], "%.1lf",
                                *(FLOAT *)szTemp);
                    }
                }
                break;
            case fldBCD:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if ((! bIsBlank) && (szTemp[0]!='\0'))
                {
                    rslt = DbiBcdToFloat((FMTBcd *)szTemp,
                            &lfFloat);
                    ChkRslt(rslt, "    BcdToFloat");

                    sprintf(pszField[uFldNum], "%.*lf",
                            pFldDescs[uFldNum].iUnits2, lfFloat);
                }
                break;
            case fldDATE:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    // Format the date. FormatDate defined in
                    // SNIPTOOL.C
                    FormatDate(*(DATE *)szTemp,
                               pszField[uFldNum]);
                }
                break;
            case fldTIME:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE)&szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    FormatTime(*(TIME *)szTemp, pszField[uFldNum]);
                }
                break;
           case fldINT16:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    sprintf(pszField[uFldNum], "%d", *(FLOAT *)szTemp);
                }
                break;
            case fldUINT16:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    sprintf(pszField[uFldNum], "%u", *(FLOAT *)szTemp);
                }
                break;
            case fldINT32:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    sprintf(pszField[uFldNum], "%ld",
                            *(INT32 *)szTemp);
                }
                break;
            case fldUINT32:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    sprintf(pszField[uFldNum], "%lu",
                            *(UINT32 *)szTemp);
                }
                break;
            case fldBOOL:
                rslt = DbiGetField(hCur, uFldNum+1, pRecBuf,
                                   (pBYTE) szTemp, &bIsBlank);
                ChkRslt(rslt, "GetField");
                if (! bIsBlank)
                {
                    if (*(BOOL*)szTemp == 0)
                    {
                        strcpy(pszField[uFldNum], "No");
                    }
                    else
                    {
                        strcpy(pszField[uFldNum], "Yes");
                    }
                }
                break;
            case fldBLOB:
                // Display the first uDefLength characters of
                // Memo BLOB fields.
                if (pFldDescs[uFldNum].iSubType == fldstMEMO)
                {
                    rslt = DbiOpenBlob(hCur, pRecBuf, (uFldNum + 1),
                                       dbiREADONLY);
                    ChkRslt(rslt, "OpenBlob");

                    //Determine the size of the BLOB
                    rslt = DbiGetBlobSize(hCur, pRecBuf, (uFldNum + 1),
                                          &ulBlobSize);
                    ChkRslt(rslt, "GetBlobSize");

                    // Determine the maximum amount to read
                    ulBlobSize = min((UINT32) ulBlobSize,
                                     (UINT32) (uDefLength - 2));

                    // Get the BLOB from the table
                    rslt = DbiGetBlob(hCur, pRecBuf, (uFldNum + 1), 0,
                                      ulBlobSize, (pBYTE) pszField[uFldNum],
                                      &ulBlobSize);
                    ChkRslt(rslt, "GetBlob");

                    // Add the NULL terminator to the string.
                    pszField[uFldNum][(UINT16)ulBlobSize] = 0;

                    // Free the blob from memory
                    rslt = DbiFreeBlob(hCur, pRecBuf, (uFldNum + 1));
                    ChkRslt(rslt, "FreeBlob");
                }
                else
                {
                    strcpy(pszField[uFldNum], "Can't Display");
                }                        
                break;
            default:
                strcpy(pszField[uFldNum], "Can't Display");
                break;
        }

        // Initialize the string in case the field was blank
        if (bIsBlank)
        {
            strcpy(pszField[uFldNum], "");
            bIsBlank = FALSE;
        }
    }
    return DBIERR_NONE;
}



DBIResult
TFileApp::FormatDate (DATE Date, pCHAR szDate)
{
    DBIResult   rslt;       // Return Value from IDAPI
    UINT16      uDay;       // Day portion of date
    UINT16      uMonth;     // Month portion of date
    INT16       iYear;      // Year portion of date
                
    // Decode the date
    rslt = DbiDateDecode(Date, &uMonth, &uDay, &iYear);
    ChkRslt(rslt, "DateDecode");

    sprintf(szDate, "%d/%d/%d", uMonth, uDay, iYear);
    return rslt;
}


DBIResult
TFileApp::FormatTime (TIME Time, pCHAR szTime)
{
   DBIResult   rslt;       // Return value from IDAPI functions
   UINT16      uHour;      // Hour portion of the time
   UINT16      uMinute;    // Minute portion of the time
   UINT16      uMilSec;    // Second portion (in ms) of the time
   UINT16      uIsAm;      // Is Time AM?
   CHAR        szTemp[10]; // Temp buffer, used for AM, PM string

   // Decode the time 
   rslt = DbiTimeDecode(Time, &uHour, &uMinute, &uMilSec);
   ChkRslt(rslt, "TimeDecode");

   // Temporary variable used to determine if the time is AM or PM
   uIsAm = uHour;
   uHour = uHour % 12;
   if (uHour == 0)
   {
       uHour = 12;
   }
   // If AM, set uIsAm to TRUE, else set uIsAm to 0
   if (uIsAm == uHour)
   {
       uIsAm = 1;
   }
   else
   {
       uIsAm = 0;
   }

   // Format the hour and minute of the time
   wsprintf(szTime, "%2u%c%02u",
            uHour,
            ':',
            uMinute);

   // Determine if seconds are to be displayed
   wsprintf(szTemp, "%c%02u",
                 ':',
                 (uMilSec / 1000));

    strcat(szTime, szTemp);
    strcat(szTime, " ");

    if (uIsAm)
    {
        strcat(szTime, "am");
    }
    else // otherwise it's PM
    {
        // Copy the PM string
        strcat(szTime, "pm");
    }

    return rslt;
}




int
OwlMain(int /*argc*/, char* /*argv*/ [])
{
  return TFileApp().Run();
}
